/*
 * FILE --- node_list.cc
 *
 * PURPOSE --- Source file for classes that implement a structure of
 * a node for a linked list and the structure of the linked list itself.
 * The node and the linked list are especially designed for an algorithm
 * for implementing a limited lookahed (planning) controller. 
 * The user specifies the dimension of the state of the plant to be controlled.
 *
 */

// Include Files
#include "node_list.hh"

// Define this macro if you want to use this utility without linking
// to the RCS Library
#ifdef NO_RCS
#include <stdio.h>            /* printf function */
#else
#include "rcs_prnt.hh"          /* rcs_print_ functions */
#endif

/*************************************************************/
OPEN_NODE::OPEN_NODE(int _plant_order)
{
  // Initialize plant parameters
  plant_order = _plant_order;

  // Allocate memory 
  allocate_memory();

  // Check and report if error occured
  if (open_node_err != OPEN_NO_ERROR)
    {
#ifdef NO_RCS
      printf("OPEN_NODE: ERROR --- Not Enought Memory!!!\n");
      printf("The node was not creatded properly ...\n");
#else
      rcs_print("OPEN_NODE: ERROR --- Not Enought Memory!!!\n");
      rcs_print("The node was not creatded properly ...\n");
#endif
    }

  // No next node at the beginning
  nextPtr = NULL;
}

OPEN_NODE::~OPEN_NODE()
{
  // Free the allocated memory
  free_memory();
}

void OPEN_NODE::allocate_memory(void)
{
  if ((state = new double[plant_order]) == NULL)
    open_node_err = OPEN_MEM_ERROR;
}

void OPEN_NODE::free_memory(void)
{
  delete [] state;
}

void OPEN_NODE::set_node(const double *_state, int _ctrl, double _cost)
{
  // Assign each element of the state
  for (int ii = 0; ii < plant_order; ii++)
    {
      state[ii] = _state[ii];
    }

  ctrl = _ctrl;
  cost = _cost;
}

void OPEN_NODE::get_state(double *_state)
{
  // Return all the elements
  for (int ii = 0; ii < plant_order; ii++)
    {
      _state[ii] = state[ii];
    }
}


OPEN_LIST::OPEN_LIST()
{
  // No nodes in the list intially
  firstPtr = NULL;
  lastPtr = NULL;
  num_of_elements = 0;
}
 

OPEN_LIST::~OPEN_LIST()
{
  // Delete all the nodes if the list is not empty
  if (!isEmpty())
    {
      open_node *tempPtr;
      open_node *currentPtr = firstPtr;

      while (currentPtr != NULL)
	{
	  tempPtr = currentPtr;
	  currentPtr = currentPtr->nextPtr;
	  delete tempPtr;
	}
    }
}


void OPEN_LIST::add_node(open_node *_new_node)
{
  // If empty then both firstPtr and lastPtr point to the new node
  if (isEmpty())
    {
      firstPtr = _new_node;
      lastPtr = _new_node;
    }
  else
    {
      _new_node->nextPtr = firstPtr;
      firstPtr = _new_node;
    }

  // The # elements increased by 1
  num_of_elements++;
}


int OPEN_LIST::remove_node(double *_state)
{
  // If empty there is no node to remove -- error 
  if (isEmpty())
    {
      // Error -- no node removed
      return (0);
    }
  else // Remove the first (top) node
    {
      open_node *tempPtr = firstPtr;
      
      if (firstPtr == lastPtr)
	{
	  firstPtr = NULL;
	  lastPtr = NULL;
	}
      else
	{
	  firstPtr = firstPtr->nextPtr;
	}

      tempPtr->get_state(_state);
      delete tempPtr;

      // The # elements decreased by 1
      num_of_elements--;

      // Success -- a node was removed
      return (1);
    }
}

int OPEN_LIST::isEmpty(void) const
{
  // Return 1=not empty, 0=empty
  return (firstPtr == NULL);
}
